from bokeh.plotting import figure, show
from bokeh.io import output_notebook


def jupytered():
    jup = False
    import __main__ as main
    if not hasattr(main, '__file__'):
        jup = True 
    return jup

def inline():
    res = False
    if jupytered():
        output_notebook()
        res = True
    return  res 
    



import sys
sys.dont_write_bytecode = True
import os
sys.path.append(os.path.dirname(__file__))
import db
import numpy as np
from datetime import date, datetime, timedelta
from bokeh.palettes import all_palettes



def PiscesDates_Offline(startDate=date(2011, 12, 31), endDate=date(2017, 12, 9)):
    delta = endDate - startDate
    dates = [(startDate + timedelta(days=x)) for x in range(0, delta.days+1, 7)]
    return dates



def nearestDate(dates, dt):
    return min(dates, key=lambda d: abs(d - dt))



def timesBetween(calTable, startDate, endDate):
    query = "SELECT [time] FROM %s WHERE " % calTable
    query += "[time] BETWEEN '%s' AND '%s' " % (startDate, endDate)
    df = db.dbFetch(query)
    return np.array(df['time'])



def temporalRes(table):
    table = table.lower()
    dt = 1                  # default temporal resolution = 1 day
    #if table.find('tblPisces'.lower()) != -1:
    #    dt = 7
    return dt



    
def getBounds(varName):
    bounds = (None, None)

    if varName.find('picoeukaryote') != -1:
        bounds = (0, 0.5)
    elif varName.find('prokaryote') != -1:
        bounds = (0, 0.5)
    elif varName.find('zooplankton') != -1:
        bounds = (0, 0.5)
    elif varName.find('POC') != -1:
        bounds = (0, 0.7)
    elif varName.find('POFe') != -1:
        bounds = (0, 8e-6)
    elif varName.find('PON') != -1:
        bounds = (0, 0.15)
    elif varName.find('POSi') != -1:
        bounds = (0, 0.15)
    elif varName.find('DOFe') != -1:
        bounds = (0, 1.3e-4)
    elif varName.find('DON') != -1:
        bounds = (0, 5)
    elif varName.find('DOP') != -1:
        bounds = (0, 0.2)
    elif varName.find('SiO2') != -1:
        bounds = (0, 30)
    elif varName.find('FeT') != -1:
        bounds = (0, 2e-4)
    elif varName.find('Fe') != -1:
        bounds = (0, 1e-4)
    elif varName.find('CDOM') != -1:
        bounds = (0, 1.3e-3)
    elif (varName.find('chl0') != -1) or (varName.find('chl1') != -1) or (varName.find('chl2') != -1) or (varName.find('chl3') != -1):
        bounds = (0, 5e-2)
    elif varName.find('chl') != -1:
        bounds = (0, 5e-1)
    elif varName.find('CHL') != -1:
        bounds = (0, 5e-1)
    elif varName.find('PHYC') != -1:
        bounds = (0, 4)
    elif varName.find('PP') != -1:
        bounds = (0, 4e-2)
    elif varName.find('Si') != -1:
        bounds = (10, 30)
    elif varName.find('NO3') != -1:
        bounds = (0, 20)
    elif varName.find('NO2') != -1:
        bounds = (0, 1.5)
    elif varName.find('NH4') != -1:
        bounds = (0, 2)
    elif varName.find('PO4') != -1:
        bounds = (0, 1.5)
    elif varName.find('O2') != -1:
        bounds = (200, 320)
    elif varName.find('ALK') != -1:
        bounds = (2000, 2400)
    elif varName.find('PIC') != -1:
        bounds = (0, 0.5)
    elif varName.find('cocco') != -1:
        bounds = (0, 0.5)
    elif varName.find('DIC') != -1:
        bounds = (1700, 2200)
    elif varName.find('DOC') != -1:
        bounds = (0, 25)
    elif varName.find('diatom') != -1:
        bounds = (0, 0.5)
    elif varName.find('diazotroph') != -1:
        bounds = (0, 0.15)
    elif varName.find('dinoflagellate') != -1:
        bounds = (0, 0.25)
    elif varName.find('wind_stress') != -1:
        bounds = (0, 3e-1)
    elif varName.find('eastward_wind') != -1:
        bounds = (0, 2)
    elif varName.find('mld_nrt') != -1:
        bounds = (0, 170)          
    elif varName.find('ftle_nrt') != -1:
        bounds = (0, 0.25)          
    elif varName.find('disp_nrt') != -1:
        bounds = (0, 2.5)          
    elif varName.find('sst') != -1:
        bounds = (0, 32)          
    elif varName.find('sla') != -1:
        bounds = (-0.3, 0.3)          
    elif varName.find('sss') != -1:
        bounds = (31, 37)          
    elif varName.find('vort') != -1:
        bounds = (-4e-6, 4e-6)          
    return bounds



'''
import matplotlib as plt
import matplotlib.cm as cm
colormap =cm.get_cmap("ocean") 
paletteName = [plt.colors.rgb2hex(m) for m in colormap(np.arange(colormap.N))]
'''


def getPalette(varName, nCols=256):
    paletteName = 'RdBu11'
    if varName.find('picoeukaryote') != -1:
        paletteName = all_palettes['Magma'][nCols]
    elif varName.find('prokaryote') != -1:
        paletteName = all_palettes['Inferno'][nCols]
    elif varName.find('zooplankton') != -1:
        paletteName = all_palettes['Plasma'][nCols]        
    elif varName.find('POC') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('POFe') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('PON') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('POSi') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('DOFe') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('DON') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('DOP') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('SiO2') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('FeT') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('Fe') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('CDOM') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('PHYC') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('PP') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('Si') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('NO3') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('NO2') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('NH4') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('PO4') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('O2') != -1:
        paletteName = all_palettes['Inferno'][nCols]
    elif varName.find('ALK') != -1:
        paletteName = all_palettes['Magma'][nCols]
    elif varName.find('PIC') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('chl') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('CHL') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('diazotroph') != -1:
        paletteName = all_palettes['Inferno'][nCols]
    elif varName.find('dinoflagellate') != -1:
        paletteName = all_palettes['Magma'][nCols]
    elif varName.find('diatom') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('cocco') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('DIC') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('DOC') != -1:
        paletteName = all_palettes['Viridis'][nCols]
    elif varName.find('wind_stress') != -1:
        paletteName = all_palettes['Plasma'][nCols]
    elif varName.find('mld_nrt') != -1:
        paletteName = all_palettes['Plasma'][nCols]        
    elif varName.find('ftle_nrt') != -1:
        paletteName = all_palettes['Inferno'][nCols]        
    elif varName.find('disp_nrt') != -1:
        paletteName = all_palettes['Inferno'][nCols]        
    elif varName.find('sst') != -1:
        paletteName = all_palettes['Inferno'][nCols]
    elif varName.find('sss') != -1:
        paletteName = all_palettes['Inferno'][11]
        # paletteName = 'RdBu11'
    elif varName.find('sla') != -1:
        paletteName = 'RdBu11'
    return paletteName



import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
import db
import subset
from datetime import datetime, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def exportData(y, table, variable, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2):
    df = pd.DataFrame()
    df[variable] = y
    timeField = 'time'
    if db.isClimatology(table) and db.hasField(table, 'month'):
        timeField = 'month'
    df['start_'+timeField] = startDate
    df['end_'+timeField] = endDate
    df['lat1'] = lat1
    df['lat2'] = lat2
    df['lon1'] = lon1
    df['lon2'] = lon2
    if db.hasField(table, 'depth'):
        df['depth1'] = depth1
        df['depth2'] = depth2
    dirPath = 'data/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    path = dirPath + 'Hist_' + table + '_' + variable + '.csv'
    df.to_csv(path, index=False)    
    return


def plotDist(tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag, marker='-', msize=30, clr='purple'):
    p = []
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables)), desc='overall'):
        #y = gd.genericDist(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, extV[i], extVV[i], extV2[i], extVV2[i])
        y = subset.spaceTime(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2) 
        y = y[variables[i]]
        if exportDataFlag:
            exportData(y, tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)
        try:    
            y = y[~np.isnan(y)]     # remove nans
        except Exception as e:
            continue    
        hist, edges = np.histogram(y, density=True, bins=50)
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        p1.yaxis.axis_label = 'Density'
        p1.xaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = variables[i]
        fill_alpha = 0.4   
        cr = p1.quad(top=hist, bottom=0, left=edges[:-1], right=edges[1:], fill_color="dodgerblue", line_color=None, hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.7, hover_line_color="white", legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='mouse'))
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Histogram")
    show(column(p))
    return


def main():
    tables = sys.argv[1].split(',')      
    variables = sys.argv[2].split(',')      
    startDate = sys.argv[3]      
    endDate = sys.argv[4]      
    #startDate = sys.argv[3].split('T')[0]      
    #endDate = sys.argv[4].split('T')[0]      
    lat1 = sys.argv[5]   
    lat2 = sys.argv[6]   
    lon1 = sys.argv[7]     
    lon2 = sys.argv[8]   
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]        
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    if float(lat1)>float(lat2):
        temp = lat1
        lat1 = lat2
        lat2 = temp

    if float(lon1)>float(lon2):
        temp = lon1
        lon1 = lon2
        lon2 = temp

    if datetime.strptime(startDate, '%Y-%m-%d')>datetime.strptime(endDate, '%Y-%m-%d'):
        temp = startDate
        startDate = endDate
        endDate = temp


    #tic = time.clock()
    plotDist(tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
    #toc = time.clock()
    #print('Fetch time: %2.2f s' % (toc-tic))


inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()


import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
import db
import timeSeries as TS
import itertools as itt
from datetime import datetime, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def exportData(t1, y1, yErr1, t2, y2, yErr2, table1, variable1, table2, variable2, lat1, lat2, lon1, lon2, depth1, depth2):
    df = pd.DataFrame()
    df['time_X'] = t1
    df[variable1] = y1
    df[variable1+'_std_X'] = yErr1
    df['time_Y'] = t2
    df[variable2] = y2
    df[variable2+'_std_Y'] = yErr2
    df['lat1'] = lat1
    df['lat2'] = lat2
    df['lon1'] = lon1
    df['lon2'] = lon2
    if db.hasField(table1, 'depth') or db.hasField(table2, 'depth'):
        df['depth1'] = depth1
        df['depth2'] = depth2
    dirPath = 'data/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    path = dirPath + 'XY_' + table1 + '_' + variable1 + '_vs_' + table2 + '_' + variable2 + '.csv'
    df.to_csv(path, index=False)    
    return

def plotXY(tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag, marker='-', msize=15, clr='green'):
    p = []
    lw = 2
    w = 500
    h = 500
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    tablePairs = list(itt.combinations(tables, 2))
    variablePairs = list(itt.combinations(variables, 2))
    for i in tqdm(range(len(tablePairs)), desc='overall'):
        t1, y1, y_std1 = TS.timeSeries(tablePairs[i][0], variablePairs[i][0], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)
        t2, y2, y_std2 = TS.timeSeries(tablePairs[i][1], variablePairs[i][1], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)
        if exportDataFlag:
            exportData(t1, y1, y_std1, t2, y2, y_std2, tablePairs[i][0], variablePairs[i][0], tablePairs[i][1], variablePairs[i][1], lat1, lat2, lon1, lon2, depth1, depth2)

        if len(t1)<1 or len(y1)<1 or len(t2)<1 or len(y2)<1:
            continue
        if (len(t1)-len(t2) != 0) or (len(y1)-len(y2) != 0):   
            continue
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        p1.xaxis.axis_label = variablePairs[i][0] + ' [' + db.getVar(tablePairs[i][0], variablePairs[i][0]).iloc[0]['Unit'] + ']'
        p1.yaxis.axis_label = variablePairs[i][1] + ' [' + db.getVar(tablePairs[i][1], variablePairs[i][1]).iloc[0]['Unit'] + ']'
        leg = variablePairs[i][0] + ' / ' + variablePairs[i][1]
        fill_alpha = 0.3       
        cr = p1.circle(y1, y2, fill_color="grey", hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.6, line_color=None, hover_line_color="white", legend=leg, size=msize)
        #p1.line(y1, y2, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))    
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="XY")
    show(column(p))
    return


def main():
    tables = sys.argv[1].split(',')      
    variables = sys.argv[2].split(',')      
    startDate = sys.argv[3]      
    endDate = sys.argv[4]      
    #startDate = sys.argv[3].split('T')[0]      
    #endDate = sys.argv[4].split('T')[0]      
    lat1 = sys.argv[5]
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]      
    lon2 = sys.argv[8]     
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]        
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    if float(lat1)>float(lat2):
        temp = lat1
        lat1 = lat2
        lat2 = temp

    if float(lon1)>float(lon2):
        temp = lon1
        lon1 = lon2
        lon2 = temp

    if datetime.strptime(startDate, '%Y-%m-%d')>datetime.strptime(endDate, '%Y-%m-%d'):
        temp = startDate
        startDate = endDate
        endDate = temp

    #tic = time.clock()
    plotXY(tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
    #toc = time.clock()
    #print('Fetch time: %2.2f s' % (toc-tic))




inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()


import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import db
import timeSeries as TS
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm


def prepareQuery(table, startDate, endDate, lat1, lat2, lon1, lon2):
    args = (table, startDate, endDate, lat1, lat2, lon1, lon2)
    query = "SELECT track, [time], lat, lon FROM %s WHERE "
    query = query + "[time]>='%s' AND [time]<='%s' AND "
    query = query + "lat>=%f AND lat<=%f AND "
    query = query + "lon>=%f AND lon<=%f "
    query = query + "ORDER BY track, [time]"
    query = query % args
    return query


def getEddies(table, startDate, endDate, lat1, lat2, lon1, lon2):
    query = prepareQuery(table, startDate, endDate, lat1, lat2, lon1, lon2)
    df = db.dbFetch(query)        
    df = pd.DataFrame.from_records(df, columns=['track', 'time', 'lat', 'lon'])    
    return df


def dumpEddyShape(lats, lons, fname):
    dirPath = 'shape/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)       
    df = pd.DataFrame()
    df['lat'] = lats
    df['lon'] = lons
    ## dump the shape file content in a csv file (this will be used by macos app)
    df.to_csv(dirPath + 'shape.csv', index=False)

    try:
        import geopandas as gpd
        from shapely.geometry import Point 
        df['geometry'] = df.apply(lambda x: Point((float(x.lon), float(x.lat))), axis=1)
        df = gpd.GeoDataFrame(df, geometry='geometry')
        df.to_file(dirPath + '%s.shp' % fname, driver='ESRI Shapefile')    
    except Exception as e:
        print('dumpEddyShape Error: ')    
        print(e)   
    return

def appendVar(track, t, y, yErr, variable):
    df = track
    df[variable] = y
    df[variable+'_std'] = yErr
    return df


def exportData(cruiseTrack, t, y, yErr, table, variable, margin):
    df = cruiseTrack
    df['margin'] = margin
    dirPath = 'data/'
    path = dirPath + 'EddyCentric.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return

def findFirstAndLast(arr, x) :
    first = -1
    last = -1
    n = len(arr)
    for i in range(0,n) :
        if (x != arr[i]) :
            continue
        if (first == -1) :
            first = i
        last = i
    return first, last     

def colocalize(tables, variables, eddy, spMargin, depth1, depth2, exportDataFlag, fname, marker='-'):
    def spectrum(ind):
        colList = ['grey', 'purple', 'darkturquoise', 'black', 'red', 'blue', 'pink', 'lime', 'green', 'orange']
        ind = ind % len(colList)
        col = colList[ind]
        return col

    fmt='%Y-%m-%d'
    dt = 24*60
    msize=10
    p = []
    loadedEddy = pd.DataFrame(eddy)
    tracks = eddy.track.unique()
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables)), desc='overall'):
        ts, ys, y_stds, track_ids = [], np.array([]), np.array([]), np.array([])
        for j in tqdm(range(len(eddy)), desc=variables[i]):
            startDate = eddy.iloc[j]['time']
            endDate = startDate
            lat1 = float(eddy.iloc[j]['lat']) - spMargin
            lat2 = float(eddy.iloc[j]['lat']) + spMargin
            lon1 = float(eddy.iloc[j]['lon']) - spMargin
            lon2 = float(eddy.iloc[j]['lon']) + spMargin           
            t, y, y_std = TS.timeSeries(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fmt=fmt, dt=dt)
            ts.append( datetime.strptime(str(eddy.iloc[j]['time']), fmt) )
            ys = np.append(ys, y[0])            
            y_stds = np.append(y_stds, y_std[0])
            track_ids = np.append(track_ids, eddy.iloc[j]['track'])
        loadedEddy = appendVar(loadedEddy, ts, ys, y_stds, variables[i])            
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Time'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = 'Eddy Centric ' + variables[i]
        fill_alpha = 0.7

        for ind in range(len(tracks)):
            track = tracks[ind]
            first, last = findFirstAndLast(track_ids, track)
            filCol = spectrum(ind)
            cr = p1.circle(ts[first:last+1], ys[first:last+1], fill_color=filCol, hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
            p1.line(ts[first:last+1], ys[first:last+1], line_color=filCol, line_width=lw, legend=leg)           
        #p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))

        p1.xaxis.formatter=DatetimeTickFormatter(
                days=["%d %B %Y"],
                months=["%d %B %Y"],
                years=["%d %B %Y"],
            )
        p1.xaxis.major_label_orientation = pi/4
        p.append(p1)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Eddy")
    show(column(p))
    if exportDataFlag:
        exportData(loadedEddy, ts, ys, y_stds, tables[i], variables[i], spMargin)   
    print('')     
    return


def main():
    eddyTable = sys.argv[1]                                     # argument1: eddy table name
    startDate = sys.argv[2]                                     # argument2: eddy cores within the delimited space-time (start date)
    endDate = sys.argv[3]                                       # argument3: eddy cores within the delimited space-time (end date)   
    lat1 = float(sys.argv[4])                                   # argument4: eddy cores within the delimited space-time (start lat)
    lat2 = float(sys.argv[5])                                   # argument5: eddy cores within the delimited space-time (end lat)               
    lon1 = float(sys.argv[6])                                   # argument6: eddy cores within the delimited space-time (start lon)
    lon2 = float(sys.argv[7])                                   # argument7: eddy cores within the delimited space-time (end lon)
    shapeFlag = bool(int(sys.argv[8]))                          # argument8: < 1 > to generate eddy trajectory shape file; < 0 > ignore 
    colocateFlag = bool(int(sys.argv[9]))                       # argument9: < 1 > to colocalize selected variables along the eddy trajectories; < 0 > ignore
    fname = sys.argv[10]                                        # argument10: figure filename (and/or shape filename)

    if shapeFlag or colocateFlag:
        cores = getEddies(eddyTable, startDate, endDate, lat1, lat2, lon1, lon2)

    # make shapefile for the eddy trajectories
    if shapeFlag:                       
        try:
            dumpEddyShape(cores.lat, cores.lon, fname)
        except Exception as e:              
            print("The following error occurred while generating the eddy shape file: ")
            print(e)

    # colocate the eddy trajectories on varialble fields
    if colocateFlag:                    
        tables = sys.argv[11].split(',')                        # argument11: comma-separated list of varaible table names                          
        variables = sys.argv[12].split(',')                     # argument12: comma-separated list of variable names
        spatialTolerance = float(sys.argv[13])                  # argument13: colocalizer spatial tolerance (+/- degrees)  
        exportDataFlag = bool(int(sys.argv[14]))                # argument14: < 1 > export the eddy trajectories and colocalized data on disk; < 0 > ignore 
        depth1 = 0
        depth2 = 5        
        colocalize(tables, variables, cores, spatialTolerance, depth1, depth2, exportDataFlag, fname, marker='-')




inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()





import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import db
import timeSeries as TS
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def prepareQuery(t, lat, lon):
    table = 'tblAltimetry_REP'
    u, v = 'ugos', 'vgos' 
    Xres, Yres = 0.25/2, 0.25/2
    args = (u, v, table, t, lat, Yres, lon, Xres, lat, lon)
    query = "SELECT [time], lat, lon, %s AS u, %s AS v FROM %s WHERE "
    query = query + "[time]='%s' AND "
    query = query + "ABS(lat-(%f))<=%f AND ABS(lon-(%f))<=%f "
    #query = query + "AND %s IS NOT NULL AND %s IS NOT NULL "
    query = query + "ORDER BY ABS(lat-(%f)), ABS(lon-(%f))"
    query = query % args
    return query


def weightedMean(df, lat, lon):
    u, v = 0, 0
    df['r'] = np.power(np.power(df['lat']-lat, 2) + np.power(df['lon']-lon, 2), 0.5)
    n = len(df)
    U, V, R = np.array(df['u']), np.array(df['v']), np.array(df['r'])  
    totalW = 0
    for i in range(n):
        if R[i] != 0:
            totalW = totalW + 1/R[i]
        else:
            return U[i], V[i]          
    for i in range(n):        
        u = u + U[i]/R[i]    
        v = v + V[i]/R[i]
    u = u/totalW    
    v = v/totalW  
    return u, v


def interpolateUV(t, lat, lon):
    query = prepareQuery(t, lat, lon)
    df = db.dbFetch(query)        
    df = pd.DataFrame.from_records(df, columns=['time', 'lat', 'lon', 'u', 'v'])
    n = len(df)
    u, v = 0, 0
    if n == 1:
        u, v = np.array(df['u'])[0], np.array(df['v'])[0]
    else:
        u, v = weightedMean(df, lat, lon)            
    if u == None:
        u = 0
    if v == None:
        v = 0
    return u, v


def propagate(direction, startDate, endDate, lat, lon, fmt, dt):
    def cond(direction, startDate, endDate, t):
        if direction==1:
            res = t<=endDate
        elif direction==-1:
            res = t>=startDate
        return res

    dx = 111180 * np.cos(lat*np.pi/180)
    dy = 111180

    ts, lats, lons = np.array([]), np.array([]), np.array([])
    startDate = datetime.strptime(startDate, fmt)
    endDate = datetime.strptime(endDate, fmt)
    if endDate < startDate:
        swap = startDate
        startDate = endDate
        endDate = swap

    if direction==1:
        t = startDate
    elif direction==-1:
        t = endDate

    ts = np.append(ts, t)
    lats = np.append(lats, lat)
    lons = np.append(lons, lon) 
    
    while cond(direction, startDate, endDate, t):  
        ##### interpolate flowfield at particle's location
        u, v = interpolateUV(t.strftime(fmt), lat, lon)
        ##### advection
        lon = lon + direction*dt*u / dx
        lat = lat + direction*dt*v / dy
        t = t + timedelta(seconds=direction*dt)
        #### update trajectory
        ts = np.append(ts, t)
        lats = np.append(lats, lat)
        lons = np.append(lons, lon)     
    return ts, lats, lons


def dumpTrackShape(lats, lons, fname):
    dirPath = 'shape/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)       
    df = pd.DataFrame()
    df['lat'] = lats
    df['lon'] = lons
    ## dump the shape file content in a csv file (this will be used by macos app)
    df.to_csv(dirPath + 'shape.csv', index=False)

    try:
        import geopandas as gpd
        from shapely.geometry import Point 
        df['geometry'] = df.apply(lambda x: Point((float(x.lon), float(x.lat))), axis=1)
        df = gpd.GeoDataFrame(df, geometry='geometry')
        df.to_file(dirPath + '%s.shp' % fname, driver='ESRI Shapefile')    
    except Exception as e:
        print('dumpTrackShape Error: ')    
        print(e)
    return

def appendVar(track, t, y, yErr, variable):
    df = track
    df[variable] = y
    df[variable+'_std'] = yErr
    return df


def exportData(cruiseTrack, t, y, yErr, table, variable, margin):
    df = cruiseTrack
    df['margin'] = margin
    dirPath = 'data/'
    path = dirPath + 'Tracer.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return


def plotAlongTrack(dt, fmt, tables, variables, track, spMargin, depth1, depth2, exportDataFlag, fname, marker='-', msize=30, clr='purple'):
    p = []
    #fmt = '%Y-%m-%d %H:%M:%S'
    dt = dt/60         # time resolution (minutes)
    loadedTrack = pd.DataFrame(track)
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables)), desc='overall'):
        ts, ys, y_stds = [], np.array([]), np.array([])
        for j in tqdm(range(len(track)), desc=variables[i]):
            startDate = track.iloc[j]['time'].strftime(fmt)
            endDate = startDate
            lat1 = float(track.iloc[j]['lat']) - spMargin
            lat2 = float(track.iloc[j]['lat']) + spMargin
            lon1 = float(track.iloc[j]['lon']) - spMargin
            lon2 = float(track.iloc[j]['lon']) + spMargin           
            t, y, y_std = TS.timeSeries(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fmt=fmt, dt=dt)
            ts.append(track.iloc[j]['time'])
            ys = np.append(ys, y[0])            
            y_stds = np.append(y_stds, y_std[0])
        loadedTrack = appendVar(loadedTrack, ts, ys, y_stds, variables[i])            
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Time'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = 'Along Tracer Track ' + variables[i]
        fill_alpha = 0.3
        cr = p1.circle(ts, ys, fill_color="grey", hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
        p1.line(ts, ys, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))
        p1.xaxis.formatter=DatetimeTickFormatter(
                hours=["%d %B %Y"],
                days=["%d %B %Y"],
                months=["%d %B %Y"],
                years=["%d %B %Y"],
            )
        p1.xaxis.major_label_orientation = pi/4
        p.append(p1)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Lagrangian")
    show(column(p))
    if exportDataFlag:
        exportData(loadedTrack, ts, ys, y_stds, tables[i], variables[i], spMargin)    
    print('')
    return





def main():
    fmt='%Y-%m-%d'
    dt = int(sys.argv[1])                               # argument1: propagation time step (day seconds)
    direction = int(sys.argv[2])                        # argument2: propagation direction (forward/backward in time)
    startDate = sys.argv[3]                             # argument3: propagation start date
    endDate = sys.argv[4]                               # argument4: propagation end date
    lat0 = float(sys.argv[5])                           # argument5: starting poin latitude
    lon0 = float(sys.argv[6])                           # argument6: starting poin longitude
    shapeFlag = bool(int(sys.argv[7]))                  # argument7: < 1 > to generate tracer trajectory shape file; < 0 > ignore           
    colocateFlag = bool(int(sys.argv[8]))               # argument8: < 1 > to colocalize selected variables along the tracer trajectory; < 0 > ignore
    fname = sys.argv[9]                                 # argument9: figure filename (and/or shape filename)

    if shapeFlag or colocateFlag:
        ts, lats, lons = propagate(direction, startDate, endDate, lat0, lon0, fmt, dt)

    # make shapefile for the tracer's trajectory
    if shapeFlag:                       
        try:            
            dumpTrackShape(lats, lons, fname)
        except Exception as e:              
            print("The following error occurred while generating the tracer shape file: ")
            print(e)


    # colocate the tracer's trajectory on varialble fields
    if colocateFlag:                    
        tables = sys.argv[10].split(',')                # argument10: comma-separated list of varaible table names                                     
        variables = sys.argv[11].split(',')             # argument11: comma-separated list of variable names
        spatialTolerance = float(sys.argv[12])          # argument12: colocalizer spatial tolerance (+/- degrees) 
        exportDataFlag = bool(int(sys.argv[13]))        # argument13: < 1 > export the tracer trajectory and colocalized data on disk; < 0 > ignore 
        depth1 = 0
        depth2 = 5        

        df = pd.DataFrame()
        df['time'] = ts
        df['lat'] = lats
        df['lon'] = lons
        plotAlongTrack(dt, fmt, tables, variables, df, spatialTolerance, depth1, depth2, exportDataFlag, fname, marker='-', msize=30, clr='darkturquoise')



inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()









import sys
sys.dont_write_bytecode = True
import os
sys.path.append(os.path.dirname(__file__))
import db
import subset
import pandas as pd
from datetime import datetime
from datetime import timedelta  
import numpy as np
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def getModelDepthLevels(tableName):
    query = "SELECT * FROM %s" % tableName
    depthLevs = db.dbFetch(query) 
    return depthLevs

def sourceQuery(srcTable):
    query  = 'SELECT * FROM %s '
    query = query % (srcTable)
    return query


def getSrcDF(DB, source):
    srcDF = None
    if DB:
        query = sourceQuery(source) 
        srcDF = db.dbFetch(query)   
    else:  
        filename, fileExtension = os.path.splitext(source)  
        if fileExtension == '.xlsx':
            srcDF = pd.read_excel(open(source,'rb'), sheet_name='data')
        elif fileExtension == '.csv':    
            srcDF = pd.read_csv(source)
        srcDF['time'] = pd.to_datetime(srcDF['time'])
    return srcDF


def matchSource(DB, source, tMargin, latMargin, lonMargin, depthMargin, tables, variables, exportPath):
    target = []
    for i in range(len(tables)):
        target.append((tables[i], variables[i]))
    srcDF = getSrcDF(DB, source)
    if not 'depth' in srcDF.columns:
        srcDF['depth'] = 0
    #depthLevs = getModelDepthLevels('tblPisces_Depth')
    #depthLevs = getModelDepthLevels('tblDarwin_Depth')
    srcDF['dt1'], srcDF['dt2'], srcDF['lat1'], srcDF['lat2'], srcDF['lon1'], srcDF['lon2'], srcDF['depth1'], srcDF['depth2'] = '', '', '', '', '', '', '', ''
    for tar in tqdm(target, desc='overall'):
        srcDF[tar[1]] = ''
        srcDF[tar[1]+'_std'] = ''
        for i in tqdm(range(len(srcDF)), desc=tar[1]):
            dt = srcDF.time[i]   
            dt1 = dt - timedelta(days=tMargin) 
            dt2 = dt + timedelta(days=tMargin) 
            lat = srcDF.lat[i]
            lon = srcDF.lon[i]
            depth = srcDF.depth[i]
            lat1, lat2 = lat - latMargin, lat + latMargin
            lon1, lon2 = lon - lonMargin, lon + lonMargin
            depth1, depth2 = depth - depthMargin, depth + depthMargin

            ##### matching to the closest model depth layer 
            ##### "depthLevs" array has to hold the model's depth levels  #####
            '''
            ind = np.abs(depthLevs['depth_level']-depth).idxmin()# argmin()
            depth = depthLevs['depth_level'][ind]
            depth1, depth2 = depth - 1, depth + 1
            '''
            ####################################################################

            data = subset.spaceTime(tar[0], tar[1], str(dt1), str(dt2), lat1, lat2, lon1, lon2, depth1, depth2)
            srcDF.loc[i, 'dt1'], srcDF.loc[i, 'dt2'], srcDF.loc[i, 'lat1'], srcDF.loc[i, 'lat2'], srcDF.loc[i, 'lon1'], srcDF.loc[i, 'lon2'], srcDF.loc[i, 'depth1'], srcDF.loc[i, 'depth2'] = dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2
            srcDF.loc[i,tar[1]] = np.mean(data[tar[1]])
            srcDF.loc[i, tar[1]+'_std'] = np.std(data[tar[1]])

    dirPath = os.path.dirname(exportPath)
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    srcDF.to_csv(exportPath, index=False)
    print('')
    return



def main():
    DB = bool(int(sys.argv[1]))                         # argument1: < 1 > if source data exists in the database. < 0 > if the source data set is a spreadsheet file on disk.
    source = sys.argv[2]                                # argument2: the source table name (or full filename)      
    temporalTolerance = float(sys.argv[3])              # argument3: colocalizer temporal tolerance (+/- degrees)
    latTolerance = float(sys.argv[4])                   # argument4: colocalizer meridional tolerance (+/- degrees)
    lonTolerance = float(sys.argv[5])                   # argument5: colocalizer zonal tolerance (+/- degrees) 
    depthTolerance = float(sys.argv[6])                 # argument6: colocalizer depth tolerance (+/- degrees)
    tables = sys.argv[7].split(',')                     # argument7: comma-separated list of varaible table names 
    variables = sys.argv[8].split(',')                  # argument8: comma-separated list of variable names   
    exportPath = sys.argv[9]                            # argument9: path to save the colocalized data set 

    matchSource(DB, source, temporalTolerance, latTolerance, lonTolerance, depthTolerance, tables, variables, exportPath)



####################################### Example ################################################
#                                                                                              #   
#   python colocalize.py 0 d:/data.csv 5 0.5 0.5 5 tblPisces_NRT NO3 d:/data_colocalized.csv   # 
#                                                                                              #
################################################################################################

if __name__ == '__main__':
    main()



from docopt import docopt
import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
import db
import subset
from common import getPalette, getBounds
from datetime import datetime, timedelta
import time
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.palettes import all_palettes
from bokeh.models import LinearColorMapper, BasicTicker, ColorBar
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def exportData(df, path):
    df.to_csv(path, index=False)    
    return


def regionalMap(tables, variabels, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    data, lats, lons, subs, frameVars = [], [], [], [], []
    for i in tqdm(range(len(tables)), desc='overall'):
        df = subset.spaceTime(tables[i], variabels[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        if len(df) < 1:
            continue

        ############### export retrieved data ###############
        if exportDataFlag:      
            dirPath = 'data/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)     
            exportPath = dirPath + 'RM_' + tables[i] + '_' + variabels[i]               
            exportData(df, path=exportPath + '.csv')
        #####################################################

        times = df[df.columns[0]].unique()  
        lats = df.lat.unique()
        lons = df.lon.unique()
        shape = (len(lats), len(lons))
       
        depths, hours =  [None], [None]
        if 'depth' in df.columns:
            depths = df.depth.unique()
        if 'hour' in df.columns:
            hours = df.hour.unique()

        unit =  ' [' + db.getVar(tables[i], variabels[i]).iloc[0]['Unit'] + ']'
        for t in times:
            for h in hours:
                for z in depths:
                    frame = df[df[df.columns[0]] == t]
                    sub = variabels[i] + unit + ', ' + df.columns[0] + ': ' + str(t) 
                    if h != None:
                        frame = frame[frame['hour'] == h]
                        sub = sub + ', hour: ' + str(h) + 'hr'
                    if z != None:
                        frame = frame[frame['depth'] == z] 
                        sub = sub + ', depth: %2.2f' % z + ' [m]'  
                    try:    
                        shot = frame[variabels[i]].values.reshape(shape)
                    except Exception as e:
                        continue    
                    data.append(shot)
                    frameVars.append(variabels[i])
                    subs.append(sub)

    bokehGM(data=data, subject=subs, fname=fname, lat=lats, lon=lons, variabels=frameVars)
    return


def canvasRect(dw, dh):
    ar = dw / dh  # aspect ratio
    h = 400 if ar > 3 else 500
    w_min = 300
    w_max = 1000
    w = int(ar * h)
    if w > w_max: w = w_max
    if w < w_min: w = w_min
    return w, h


def bokehGM(data, subject, fname, lat, lon, variabels):
    TOOLS="hover,crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    w, h = canvasRect(dw=np.max(lon)-np.min(lon), dh=np.max(lat)-np.min(lat))
    p = []
    for ind in range(len(data)):
        p1 = figure(tools=TOOLS, toolbar_location="left", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lon), np.max(lon)), y_range=(np.min(lat), np.max(lat)))
        p1.xaxis.axis_label = 'Longitude'
        p1.yaxis.axis_label = 'Latitude'
        bounds = getBounds(variabels[ind])
        paletteName = getPalette(variabels[ind])
        if bounds[0]==None:
            color_mapper = LinearColorMapper(palette=paletteName)
        else:   
            color_mapper = LinearColorMapper(palette=paletteName, low=bounds[0], high=bounds[1])     
        p1.image(image=[data[ind]], color_mapper=color_mapper, x=np.min(lon), y=np.min(lat), dw=np.max(lon)-np.min(lon), dh=np.max(lat)-np.min(lat))
        color_bar = ColorBar(color_mapper=color_mapper, ticker=BasicTicker(),
                        label_standoff=12, border_line_color=None, location=(0,0))
        p1.add_layout(color_bar, 'right')
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        

    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Regional Map")
    show(column(p))
    return




def main():
    tables = sys.argv[1]      
    variables = sys.argv[2]      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]      
    lat1 = sys.argv[5]      
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]      
    lon2 = sys.argv[8]      
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]      
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    regionalMap(tables.split(','), variables.split(','), dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)


inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()
    




import platform
import sys
sys.dont_write_bytecode = True
import pyodbc
import pandas.io.sql as sql
from pandas import DataFrame


def dbConnect(usr='ArmLab', psw='ArmLab2018', ip='128.208.239.15', port='1433', db='Opedia', TDS_Version='7.3'):
    try:
        server = ip + ',' + port
        if platform.system().lower().find('windows') != -1:
            # conn = pyodbc.connect(DRIVER='{SQL Server}', SERVER=server, DATABASE=db, Uid=usr, Pwd=psw )
            conn = pyodbc.connect(DRIVER='{SQL Server}', SERVER='tcp:'+ip, PORT=port, DATABASE=db, Uid=usr, Pwd=psw )
            # conn = pyodbc.connect(DSN='cmap', SERVER='tcp:'+ip, PORT=port, DATABASE=db, Uid=usr, Pwd=psw )
        elif platform.system().lower().find('darwin') != -1:
            conn = pyodbc.connect(DRIVER='/usr/local/lib/libtdsodbc.so', TDS_Version=TDS_Version, server=ip , port=port, DATABASE=db, Uid=usr, Pwd=psw )
        elif platform.system().lower().find('linux') != -1:
            conn = pyodbc.connect(DRIVER='/usr/lib/x86_64-linux-gnu/odbc/libtdsodbc.so', TDS_Version=TDS_Version, server=ip , port=port, DATABASE=db, uid=usr, pwd=psw)
        #print('Successful database connection')
    except Exception as e:
        print('Database connection error. Error message: '+str(e))        
    return conn


def dbFetch(query):
    conn = dbConnect()
    df = sql.read_sql(query, conn)
    conn.close()
    return df


def dbFetchStoredProc(query, args):
    conn = dbConnect()
    cur = conn.cursor()
    if sys.version_info[0] >= 3:     # if python3 
        args = [ str(a) if a is not None else a for a in args ]
    cur.execute(query, args)
    df = cur.fetchall()
    cols = [column[0] for column in cur.description]
    df = DataFrame.from_records(df, columns=cols)
    conn.close()
    return df


def bulkInsert(filePath, tableName):
    conn = dbConnect()
    cursor = conn.cursor()
    query = "BULK INSERT %s FROM '%s' WITH ( FIELDTERMINATOR = ',', ROWTERMINATOR = '\n', FIRSTROW = 2 ) " % (tableName, filePath)
    cursor.execute(query, [])
    conn.commit()
    return

    
def getVar(tableName, varName):
    query = "SELECT * FROM tblVariables WHERE Table_Name='%s' AND Short_Name='%s'" % (tableName, varName)
    df = dbFetch(query)
    return df

def getTableName(varName):
    ## 
    query = "SELECT * FROM tblVariables WHERE Table_Name='%s' AND Short_Name='%s'" % (tableName, varName)
    df = dbFetch(query)
    return df

def hasField(tableName, field):
    query = "SELECT COL_LENGTH('%s', '%s') AS RESULT " % (tableName, field)
    df = dbFetch(query)
    return False if df['RESULT'][0] == None else True


def isClimatology(tableName):
    return True if tableName.find('_Climatology') != -1 else False    




import sys
import os
sys.path.append(os.path.dirname(__file__))
import db
import numpy as np
import pandas as pd
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
from bokeh.models import Legend
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm




def exportData(df):
    dirPath = 'data/'
    path = dirPath + 'esv.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return


def aggQuery(depth1, depth2, cruise_name, cluster_level, size_frac_lower, size_frac_upper, domain, kingdom, phylum, class_tax, order, genus, species):
    query = "SELECT [time], lat, lon, AVG(depth) AS depth, SUM(relative_abundance) AS abund FROM tblESV WHERE "
    query += "depth BETWEEN %f AND %f AND " % (depth1, depth2)
    query += "cruise_name='%s' AND " % cruise_name
    query += "cluster_level=%d AND " % cluster_level  
    query += "size_frac_lower=%f AND " % size_frac_lower  
    if size_frac_upper is not None:
        query += "size_frac_upper=%f AND " % size_frac_upper 
    else:
        query += "size_frac_upper IS NULL AND " 

    if domain is not None:
        if domain == nullPlaceholder:
            query = query + "RTRIM(LTRIM(domain)) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM(domain))='%s' AND " % domain
    if kingdom is not None:
        if kingdom == nullPlaceholder:
            query = query + "RTRIM(LTRIM(kingdom)) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM(kingdom)) = '%s' AND " % kingdom 
    if phylum is not None:
        if phylum == nullPlaceholder:
            query = query + "RTRIM(LTRIM(phylum)) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM(phylum)) = '%s' AND " % phylum
    if class_tax is not None:
        if class_tax == nullPlaceholder:
            query = query + "RTRIM(LTRIM([class])) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM([class])) = '%s' AND " % class_tax
    if order is not None:
        if order == nullPlaceholder:
            query = query + "RTRIM(LTRIM([order])) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM([order])) = '%s' AND " % order
    if genus is not None:
        if genus == nullPlaceholder:
            query = query + "RTRIM(LTRIM([genus])) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM([genus])) = '%s' AND " % genus
    if species is not None:
        if genus == nullPlaceholder:
            query = query + "RTRIM(LTRIM([species])) IS NULL AND "
        else:    
            query = query + "RTRIM(LTRIM([species])) = '%s' AND " % species
    query = query[:-4]
    query = query + "GROUP BY [time], [lat], [lon] "
    query = query + "ORDER BY [time], [lat], [lon] "
    df = db.dbFetch(query)
    return df


def organismsList(tax, depth1, depth2, cruise_name, cluster_level, size_frac_lower, size_frac_upper, table='tblESV'):
    query = "SELECT [%s], SUM(relative_abundance) AS total FROM %s WHERE " % (tax, table)
    query += "depth BETWEEN %f AND %f AND " % (depth1, depth2)
    query += "cruise_name='%s' AND " % cruise_name
    query += "cluster_level=%d AND " % cluster_level  
    query += "size_frac_lower=%f AND " % size_frac_lower  
    if size_frac_upper is not None:
        query += "size_frac_upper=%f " % size_frac_upper 
    else:
        query += "size_frac_upper IS NULL " 
    query += "GROUP BY [%s] ORDER BY total DESC" % tax
    df = db.dbFetch(query)
    df[tax] = df[tax].str.strip()
    return df





def plotESVs(topN, tax, depth1, depth2, cruise_name, cluster_level, size_frac_lower, size_frac_upper):
    def spectrum(ind):
        colList = ['grey', 'purple', 'darkturquoise', 'black', 'red', 'blue', 'pink', 'lime', 'green', 'orange']
        ind = ind % len(colList)
        col = colList[ind]
        return col

    
    # get the list of organisms at the specified taxonomy level, "tax", sorted by their abundance 
    orgs = organismsList(tax, depth1, depth2, cruise_name, cluster_level, size_frac_lower, size_frac_upper)
    if topN < len(orgs):
        orgs = orgs[:topN]
        
    orgs = np.array(orgs[tax])  

    msize=10
    p = []
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
    p1.xaxis.axis_label = 'Latitude'
    p1.yaxis.axis_label = 'Relative Abundance'
    fill_alpha = 0.7

    df = pd.DataFrame()
    for i in tqdm(range(len(orgs)), desc='overall'):
        if orgs[i] is None:
            orgs[i] = nullPlaceholder
        domain, kingdom, phylum, class_tax, order, genus, species = None, None, None, None, None, None, None
        if tax.lower().find('domain') != -1:
            domain = orgs[i]
        elif tax.lower().find('kingdom') != -1:    
            kingdom = orgs[i]
        elif tax.lower().find('phylum') != -1:    
            phylum = orgs[i]
        elif tax.lower().find('class') != -1:    
            class_tax = orgs[i]
        elif tax.lower().find('order') != -1:    
            order = orgs[i]
        elif tax.lower().find('genus') != -1:    
            genus = orgs[i]
        elif tax.lower().find('species') != -1:    
            species = orgs[i]


        dfOrg = aggQuery(depth1, depth2, cruise_name, cluster_level, size_frac_lower, size_frac_upper, domain, kingdom, phylum, class_tax, order, genus, species)
        x, y = dfOrg['lat'], dfOrg['abund']        
        leg = orgs[i]
        filCol = spectrum(i)
        cr = p1.circle(x, y, fill_color=filCol, hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg , size=msize)
        p1.line(x, y, line_color=filCol, line_width=lw, legend=leg )          

        if i == 0:
            df = dfOrg.copy()
            df.drop(['abund'], axis=1, inplace=True)
        df[orgs[i]+'_abund'] = dfOrg['abund']


    p.append(p1)
    
    if exportDataFlag:
        exportData(df)   

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Eddy")
    show(column(p))
    print('')   
    
    return





inline = jup.inline()   # check if jupyter is calling this script
nullPlaceholder = 'NULL'
exportDataFlag = True
fname = 'esv'





import os
import pandas as pd
import sys
sys.path.append(os.path.dirname(os.path.realpath(__file__)))
import db


def catQuery():
    query = ""    
    query = query + "SELECT RTRIM(LTRIM(Short_Name)) AS Variable, "
    query = query + "RTRIM(LTRIM(Long_Name)) AS [Long Name], "
    query = query + "RTRIM(LTRIM(Unit)) AS Unit, "
    query = query + "RTRIM(LTRIM(Make)) AS Make, "
    query = query + "RTRIM(LTRIM(Sensor)) AS Sensor, "
    query = query + "RTRIM(LTRIM(Process_Stage_Long)) AS [Process Level], "
    query = query + "RTRIM(LTRIM(Study_Domain)) AS [Study Domain], "
    query = query + "RTRIM(LTRIM(Temporal_Resolution)) AS [Temporal Resolution], "
    query = query + "RTRIM(LTRIM(Spatial_Resolution)) AS [Spatial Resolution], "
    query = query + "RTRIM(LTRIM(Comment)) AS [Comment], "

    query = query + "RTRIM(LTRIM(Dataset_Long_Name)) AS [Dataset Name], "
    query = query + "RTRIM(LTRIM(Data_Source)) AS [Data Source], "
    query = query + "RTRIM(LTRIM(Distributor)) AS [Distributor], "
    query = query + "RTRIM(LTRIM(Description)) AS [Dataset Description], "
    query = query + "[tblVariables].Dataset_ID AS [Dataset_ID], "
    query = query + "[tblVariables].ID AS [ID], "
    query = query + "[tblVariables].Table_Name AS [Table_Name], "
    query = query + "[tblVariables].Keywords AS [Keywords] "
    
    query = query + "FROM tblVariables "
    query = query + "JOIN tblDatasets ON [tblVariables].Dataset_ID=[tblDatasets].ID "
    query = query + "JOIN tblTemporal_Resolutions ON [tblVariables].Temporal_Res_ID=[tblTemporal_Resolutions].ID "
    query = query + "JOIN tblSpatial_Resolutions ON [tblVariables].Spatial_Res_ID=[tblSpatial_Resolutions].ID "
    query = query + "JOIN tblMakes ON [tblVariables].Make_ID=[tblMakes].ID "
    query = query + "JOIN tblSensors ON [tblVariables].Sensor_ID=[tblSensors].ID "
    query = query + "JOIN tblProcess_Stages ON [tblVariables].Process_ID=[tblProcess_Stages].ID "
    query = query + "JOIN tblStudy_Domains ON [tblVariables].Study_Domain_ID=[tblStudy_Domains].ID "
    return query




def exportData(df, path):
    df.to_csv(path, index=False, encoding='utf-8')    
    return

def catalog(dirPath):   
    query = catQuery()
    df = db.dbFetch(query)
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)                
    exportData(df, path=dirPath + 'catalog.csv')
    return



dirPath = 'data/'
catalog(dirPath)


import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
import db
import common as com
import climatology as clim
import subset
from datetime import datetime, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def iterative(table):
    table = table.lower()
    it = False
    if table.find('tblDarwin'.lower()) != -1:
        it = True
    if table.find('tblPisces'.lower()) != -1:
        it = True
    return it


def exportData(z, y, yErr, table, variable, date1, date2, lat1, lat2, lon1, lon2, fname):
    df = pd.DataFrame()    
    df['depth'] = z
    df[variable] = y
    df[variable+'_std'] = yErr
    if db.isClimatology(table):
        df['month1'] = clim.timeToMonth(date1)  
        df['month2'] = clim.timeToMonth(date2)  
    else:
        df['time1'] = date1
        df['time2'] = date2
    df['lat1'] = lat1
    df['lat2'] = lat2
    df['lon1'] = lon1
    df['lon2'] = lon2
    dirPath = 'data/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    path = dirPath + fname + '_' + table + '_' + variable + '.csv'
    df.to_csv(path, index=False)    
    return


def depthProfile_iterative(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    if db.isClimatology(table) and db.hasField(table, 'month'):
        m1 = clim.timeToMonth(dt1)
        m2 = clim.timeToMonth(dt2)
        if m2>m1:
            timesteps = range(m1, m2+1)
        else:
            timesteps = range(m2, m1+1)
        timesteps = ['2016-%2.2d-01' % m for m in timesteps]    
    elif table.lower().find('tblPisces'.lower()) != -1:        # currently (Nov 2018) only Pisces table has a calendar table. all datasets have to have a calendar  table. we can then remove thw else: clause below 
        calTable = table+'_Calendar'
        timesteps = com.timesBetween(calTable, dt1, dt2)   
    else:        
        delta = datetime.strptime(dt2, '%Y-%m-%d') - datetime.strptime(dt1, '%Y-%m-%d')
        timesteps = [(datetime.strptime(dt1, '%Y-%m-%d') + timedelta(days=x)).strftime('%Y-%m-%d') for x in range(delta.days+1)]

    zs, ys, y_stds = [], [], []
    for dt in timesteps:
        df = subset.depthProfile(table, field, dt, dt, lat1, lat2, lon1, lon2, depth1, depth2)
        if len(df[field]) < 1:
            continue
        zs.append(df['depth'])
        ys.append(df[field])
        y_stds.append(df[field + '_std'])

    depth = np.mean( np.stack(zs, axis=0), axis=0 )
    y = np.mean( np.stack(ys, axis=0), axis=0 )
    y_std = np.mean( np.stack(y_stds, axis=0), axis=0 )

    if exportDataFlag:
        exportData(depth, y, y_std, table, field, dt1, dt2, lat1, lat2, lon1, lon2, fname)    
    return depth, y, y_std


def depthProfile(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    df = subset.depthProfile(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
    if exportDataFlag:
        exportData(df['depth'], df[field], df[field + '_std'], table, field, dt1, dt2, lat1, lat2, lon1, lon2, fname)    
    return df['depth'], df[field], df[field + '_std']


def plotDepthProfile(tables, variables, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag, marker='-', msize=25, clr='orangered'):
    p = []
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables)), desc='overall'):
        if not db.hasField(tables[i], 'depth'):
            continue
        if not iterative(tables[i]):    
            depths, y, yErr = depthProfile(tables[i], variables[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
        else:    
            depths, y, yErr = depthProfile_iterative(tables[i], variables[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Depth'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = variables[i]
        cr = p1.circle(depths, y, fill_color="grey", hover_fill_color="firebrick", fill_alpha=0.25, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
        p1.line(depths, y, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Depth Profile")
    show(column(p))
    return


def main():
    tables = sys.argv[1].split(',')      
    variables = sys.argv[2].split(',')      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]
    lat1 = sys.argv[5]     
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]     
    lon2 = sys.argv[8]     
    depth1 = sys.argv[9]   
    depth2 = sys.argv[10]    
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    if float(lat1)>float(lat2):
        temp = lat1
        lat1 = lat2
        lat2 = temp

    if float(lon1)>float(lon2):
        temp = lon1
        lon1 = lon2
        lon2 = temp

    if float(depth1)>float(depth2):
        temp = depth1
        depth1 = depth2
        depth2 = temp

    #tic = time.clock()
    plotDepthProfile(tables, variables, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
    #toc = time.clock()
    #print('Fetch time: %2.2f s' % (toc-tic))


inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()   


import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import db
import timeSeries as TS
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def plot_single_hist(data, clr='m', labelx='', labely='', leg='', yscale='linear', store_path='', bincount=50):   
    if len(filter(None, data)) < 1:
        return
    if np.isnan(np.nansum(data)):   # if all values are nans
        return
    plt.clf()	
    bins = np.linspace(np.nanmin(data), np.nanmax(data), bincount)
    lw = 1

    ss = len(data)
    data=np.nan_to_num(data)

    plt.hist(data, bins=bins, color=clr, histtype='step', linewidth=lw, alpha=1)
    plt.hist(data, bins=bins, color=clr, label=[leg + str(', Sample Size: ') + str(ss)], histtype='stepfilled', alpha=0.25)
    if len(labelx) > 0:
        plt.xlabel(labelx)
    if len(labely) > 0:
        plt.ylabel(labely)
    if len(leg) > 0:
        le = plt.legend()
        le.set_frame_on(False)
    plt.gca().set_yscale(yscale)    
    plt.axis([None, None, 0, None])
    plt.tight_layout()
    plt.show(block=True)
    #if len(store_path) > 0:
    #    plt.savefig(store_path, format='png', dpi=300, transparent=plot_transparency)
    return


def prepareQuery(table, startDate, endDate, lat1, lat2, lon1, lon2, ftleField, ftleValue):
    args = (table, startDate, endDate, lat1, lat2, lon1, lon2, ftleField, ftleValue)
    query = "SELECT [time], lat, lon FROM %s WHERE "
    query = query + "[time]>='%s' AND [time]<='%s' AND "
    query = query + "lat>=%f AND lat<=%f AND "
    query = query + "lon>=%f AND lon<=%f AND "
    query = query + "%s>=%f "
    query = query + "ORDER BY [time], lat, lon"
    query = query % args
    return query


def getFronts(table, startDate, endDate, lat1, lat2, lon1, lon2, ftleField, ftleValue):
    query = prepareQuery(table, startDate, endDate, lat1, lat2, lon1, lon2, ftleField, ftleValue)
    df = db.dbFetch(query)        
    df = pd.DataFrame.from_records(df, columns=['time', 'lat', 'lon'])    
    return df


def match(geomTable, bkgTable, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, ftleField, ftleValue, bkgField, margin, bkgFlag):       
    query = 'EXEC uspftleMatch ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
    args = [geomTable, ftleField, str(ftleValue), bkgTable, bkgField, startDate, endDate, str(lat1), str(lat2), str(lon1), str(lon2), str(depth1), str(depth2), str(margin), bkgFlag]
    df = db.dbFetchStoredProc(query, args)
    return df

def match_temptable(geomTable, bkgTable, startDate, lat1, lat2, lon1, lon2, ftleField, ftleValue, bkgField, margin):
    args = (geomTable, startDate, lat1, lat2, lon1, lon2, ftleField, ftleValue, bkgTable, startDate, lat1, lat2, lon1, lon2, bkgField, margin, margin)
    query = ''
    query = query + "DROP TABLE IF EXISTS #tblGeometry; "
    query = query + "DROP TABLE IF EXISTS #tblBkg; "
    query = query + "SELECT * INTO #tblGeometry FROM %s WHERE [time]='%s' AND lat>=%f AND lat<=%f AND lon>=%f AND lon<=%f AND %s>=%f; "
    query = query + "SELECT * INTO #tblBkg FROM %s WHERE [time]='%s' AND lat>=%f AND lat<=%f AND lon>=%f AND lon<=%f; "
    query = query + "SELECT [#tblBkg].[time], [#tblBkg].lat, [#tblBkg].lon, [#tblBkg].%s FROM #tblBkg WHERE EXISTS "
    query = query + "(SELECT [#tblGeometry].[time], [#tblGeometry].lat, [#tblGeometry].lon FROM #tblGeometry WHERE "
    query = query + "[#tblGeometry].[time]=[#tblBkg].[time] AND ABS([#tblGeometry].lat-([#tblBkg].lat))<=%f AND ABS([#tblGeometry].lon-([#tblBkg].lon))<=%f)"
    query = query % args
    df = db.dbFetch(query)        
    df = pd.DataFrame.from_records(df, columns=['time', 'lat', 'lon'])    
    return df


def dumpFrontShape(lats, lons, fname):
    dirPath = 'shape/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)       
    df = pd.DataFrame()
    df['lat'] = lats
    df['lon'] = lons
    ## dump the shape file content in a csv file (this will be used by macos app)
    df.to_csv(dirPath + 'shape.csv', index=False)

    try:
        import geopandas as gpd
        from shapely.geometry import Point 
        df['geometry'] = df.apply(lambda x: Point((float(x.lon), float(x.lat))), axis=1)
        df = gpd.GeoDataFrame(df, geometry='geometry')
        df.to_file(dirPath + '%s.shp' % fname, driver='ESRI Shapefile')    
    except Exception as e:
        print('dumpFrontShape Error: ')    
        print(e)    
    return

def appendVar(track, t, y, yErr, variable):
    df = track
    df[variable] = y
    df[variable+'_std'] = yErr
    return df


def exportData(cruiseTrack, t, y, yErr, table, variable, margin):
    df = cruiseTrack
    df['margin'] = margin
    dirPath = 'data/'
    path = dirPath + 'Front.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return

def colocalize(ftleTable, ftleField, ftleValue, tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, spMargin, exportDataFlag, fname, bkgComparison, marker='-'):
    fmt='%Y-%m-%d'
    dt = 24*60
    msize=10
    p = []
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables)), desc='overall'):
        df = match(ftleTable, tables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, ftleField, ftleValue, variables[i], spMargin, '0')
        if len(df)<1:
            continue

        if i==0:
            loadedFTLE = pd.DataFrame(df)    
        ts, ys, y_stds = df[df.columns[0]], df[variables[i]], ''
        if i>0:
            loadedFTLE = appendVar(loadedFTLE, ts, ys, y_stds, variables[i]) 
        #plot_single_hist(ys, clr='m', labelx='', labely='', leg='', yscale='linear', store_path='', bincount=50)

        ys = ys[~np.isnan(ys)]     # remove nans
        hist, edges = np.histogram(ys, density=False, bins=50)
        if bkgComparison:
            ########## get backgrounf distribution (not matched with fronts)
            dfBkg = match(ftleTable, tables[i], startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, ftleField, ftleValue, variables[i], spMargin, '1')
            ysBkg = dfBkg[variables[i]]
            ysBkg = ysBkg[~np.isnan(ysBkg)]     # remove nans
            histBkg, edgesBkg = np.histogram(ysBkg, density=False, bins=50)
            ################################################################
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        p1.yaxis.axis_label = 'Density'
        p1.xaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = variables[i]
        fill_alpha = 0.4   
        cr = p1.quad(top=hist, bottom=0, left=edges[:-1], right=edges[1:], fill_color="dodgerblue", line_color=None, hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.7, hover_line_color="white", legend=leg + ' on fronts')
        if bkgComparison:        
            cr = p1.quad(top=histBkg, bottom=0, left=edgesBkg[:-1], right=edgesBkg[1:], fill_color="purple", line_color=None, hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.7, hover_line_color="white", legend='background')        
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='mouse'))
        p.append(p1)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Front")
    show(column(p))
    if exportDataFlag:
        exportData(loadedFTLE, ts, ys, y_stds, tables[i], variables[i], spMargin)    
    print('')
    return df













def main():
    ftleTable = sys.argv[1]                                     # argument1: ftle table name
    ftleField = sys.argv[2]                                     # argument2: ftle (type) field name
    ftleValue = float(sys.argv[3])                              # argument3: lower bound ftle value
    bkgComparison = bool(int(sys.argv[4]))                      # argument4: < 1 > to compare with the backgound values
    startDate = sys.argv[5]                                     # argument5: within the delimited space-time (start date)    
    endDate = sys.argv[6]                                       # argument6: within the delimited space-time (end date) 
    lat1 = float(sys.argv[7])                                   # argument7: within the delimited space-time (start lat) 
    lat2 = float(sys.argv[8])                                   # argument8: within the delimited space-time (end lat)     
    lon1 = float(sys.argv[9])                                   # argument9: within the delimited space-time (start lon) 
    lon2 = float(sys.argv[10])                                  # argument10: within the delimited space-time (end lon)     
    shapeFlag = bool(int(sys.argv[11]))                         # argument11: < 1 > to generate ftle shape file; < 0 > ignore 
    colocateFlag = bool(int(sys.argv[12]))                      # argument12: < 1 > to colocalize selected variables along the ftle ridges; < 0 > ignore
    fname = sys.argv[13]                                        # argument13: figure filename (and/or shape filename)


    if shapeFlag or colocateFlag:
        cores = getFronts(ftleTable, startDate, endDate, lat1, lat2, lon1, lon2, ftleField, ftleValue)


    # make shapefile for the ftle ridges
    if shapeFlag:                       
        try:
            dumpFrontShape(cores.lat, cores.lon, fname)
        except Exception as e:              
            print("The following error occurred while generating the ftle shape file: ")
            print(e)


    # colocate the ftle ridges on varialble fields
    if colocateFlag:                   
        tables = sys.argv[14].split(',')                        # argument14: comma-separated list of varaible table names
        variables = sys.argv[15].split(',')                     # argument15: comma-separated list of variable names  
        spatialTolerance = float(sys.argv[16])                  # argument16: colocalizer spatial tolerance (+/- degrees)
        exportDataFlag = bool(int(sys.argv[17]))                # argument17: < 1 > export the ftle ridges and colocalized data on disk; < 0 > ignore 
        depth1 = 0
        depth2 = 5

        df = colocalize(ftleTable, ftleField, ftleValue, tables, variables, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, spatialTolerance, exportDataFlag, fname, bkgComparison, marker='-')
        
        try:
            dumpFrontShape(df.lat, df.lon, fname)
        except Exception as e:              
            print("The following error occurred while generating the colocalized ftle shape file: ")
            print(e)


inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()










import os
import sys
sys.path.append(os.path.dirname(__file__))
import dropbox


def makedir(directory):
    	if not os.path.exists(directory):
		os.makedirs(directory)
	return

def vaultUpload(token, source, destination):
    dbx = dropbox.Dropbox(token)
    with open(source, 'rb') as f:
        dbx.files_upload(f.read(), destination)
    return


def listDBXContent(path):
    dbx = dropbox.Dropbox(token)
    f = open('.struc.txt', 'w')
    for entry in dbx.files_list_folder(path).entries:
        f.write(entry.name+'\n')
    f.close()    
    return        


def exportStruc(token):
    dbx = dropbox.Dropbox(token)
    makedir(vaultStrucDir)
    items=[]
    for entry in dbx.files_list_folder('', recursive=True).entries:
        items.append(entry.path_display)
        makedir(vaultStrucDir+entry.path_display)

    for item in items:
        path, dirs, files = os.walk(vaultStrucDir+item).next()
        count = len(files) + len(dirs)
        if item.split('/')[-1] in ['code', 'doc', 'nrt', 'rep'] and count<2000:
            for entry in dbx.files_list_folder(item, recursive=False).entries:            
                makedir(vaultStrucDir+entry.path_display)           
    return        

def refreshFolder(folder):
    dbx = dropbox.Dropbox(token)
    for entry in dbx.files_list_folder(folder, recursive=False).entries:            
        makedir(vaultStrucDir+entry.path_display)           
    
    return

vaultStrucDir = '.struc/'
token = 'dO4RNV4OBNAAAAAAAAFyg3gxT_Wu3zsPPitxxcmLi0_cas-3GF6-vqM_rJ0ioe3o'

if len(sys.argv)<2:
    exportStruc(token)

if len(sys.argv)>2:
    source = sys.argv[1]
    destination = sys.argv[2]
    vaultUpload(token, source, destination)
    refreshFolder(os.path.dirname(destination))



import sys
import os
sys.path.append(os.path.dirname(__file__))
sys.dont_write_bytecode = True
import numpy as np
import pandas as pd
import db
import subset
from datetime import datetime, timedelta
import time


def fillGaps(ts_original, y_original, y_std_original, startDate, endDate, fmt, dt):
    y = np.array([])
    y_std = np.array([])
    ts = []    
    startDate = datetime.strptime(str(startDate), fmt)
    endDate = datetime.strptime(str(endDate), fmt)
    t = startDate
    i, ind = 0, 0
    while t<=endDate:                        
        ts.append(t)
        gap = False
        if ind >= len(ts_original):
            gap = True
        elif ts_original[ind] == ts[i]:
            y = np.append(y, y_original[ind])
            y_std = np.append(y_std, y_std_original[ind])
            ind += 1
        else:
            gap = True
            
        if gap:
            y = np.append(y, np.nan)
            y_std = np.append(y_std, np.nan)            
        t = t + timedelta(minutes=dt)
        i += 1
    return ts, y, y_std


def iterative(table, field, dt):
    table = table.lower()
    it = False
    if dt != 24*60:
        it = True
    #if table.find('tblWind'.lower()) != -1:
    #    it = True
    #if table.find('tblCHL_OI'.lower()) != -1:
    #    it = True   
    #if table.find('tblPisces'.lower()) != -1:
    #    it = True
    return it


def timeSeries_iterative(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fmt='%Y-%m-%d', dt=24*60):
    y = np.array([])
    y_std = np.array([])
    ts = []    
    startDate = datetime.strptime(startDate, fmt)
    endDate = datetime.strptime(endDate, fmt)
    t = startDate
    while t<=endDate:        
        ts.append(t)
        t1 = t
        t2 = t + timedelta(minutes=dt) + timedelta(seconds=-1)
        df = subset.spaceTime(table, field, t1, t2, lat1, lat2, lon1, lon2, depth1, depth2)  
        t = t + timedelta(minutes=dt)
        try:
            if len(df[field]) > 0:                
                tempY = np.nanmean(df[field])
            else:
                tempY = np.nan
        except:
            tempY = np.nan   

        if abs(tempY) > 1e30:       ## remove outliers (extremes)
            tempY = np.nan   
            
        y = np.append(y, tempY)

        try:
            if len(df[field]) > 0:
                tempY_std = np.nanstd(df[field])
            else:
                tempY_std = np.nan
        except:
            tempY_std = np.nan        

        if abs(tempY_std) > 1e30:       ## remove outliers (extremes)
            tempY_std = np.nan   

        y_std = np.append(y_std, tempY_std)
    return ts, y, y_std


def timeSeries(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fmt='%Y-%m-%d', dt=24*60):    
    if iterative(table, field, dt):
        ts, y, y_std = timeSeries_iterative(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2, fmt, dt)
    else:   
        df = subset.timeSeries(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)

        if table.lower().find('tblseaflow') != -1:
            from plotCruise import resample
            df = resample(df, 'D', removeNAs=False)
            df[field+'_std'] = None

        if not db.isClimatology(table):
            ts, y, y_std = pd.to_datetime(df[df.columns[0]]), df[field], df[field+'_std']
            ts, y, y_std = fillGaps(ts, y, y_std, startDate, endDate, fmt, dt)
        else:    
            ts, y, y_std = df[df.columns[0]], df[field], df[field+'_std']
    return ts, y, y_std




import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
from scipy.interpolate import griddata
import db
import subset
from common import getPalette, getBounds
import climatology as clim
from datetime import datetime, timedelta
import time
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.palettes import all_palettes
from bokeh.models import LinearColorMapper, BasicTicker, ColorBar
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def exportData(df, path):
    df.to_csv(path, index=False)    
    return


def sectionMap(tables, variabels, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    data, lats, lons, subs, frameVars = [], [], [], [], []
    xs, ys, zs = [], [], []
    for i in tqdm(range(len(tables)), desc='overall'):
        if not db.hasField(tables[i], 'depth'):
            continue        
        df = subset.section(tables[i], variabels[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        if len(df) < 1:
            continue

        ############### export retrieved data ###############
        if exportDataFlag:      # export data
            dirPath = 'data/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)                
            exportData(df, path=dirPath + fname + '_' + tables[i] + '_' + variabels[i] + '.csv')
        #####################################################

        times = df[df.columns[0]].unique() 
        lats = df.lat.unique()
        lons = df.lon.unique()
        depths = df.depth.unique()
        shape = (len(lats), len(lons), len(depths))

        hours =  [None]
        if 'hour' in df.columns:
            hours = df.hour.unique()

        unit =  ' [' + db.getVar(tables[i], variabels[i]).iloc[0]['Unit'] + ']'

        for t in times:
            for h in hours:
                frame = df[df[df.columns[0]] == t]
                sub = variabels[i] + unit + ', ' + df.columns[0] + ': ' + str(t) 
                if h != None:
                    frame = frame[frame['hour'] == h]
                    sub = sub + ', hour: ' + str(h) + 'hr'
                try:    
                    shot = frame[variabels[i]].values.reshape(shape)
                except Exception as e:
                    continue    
                data.append(shot)
                
                xs.append(lons)
                ys.append(lats)
                zs.append(depths)

                frameVars.append(variabels[i])
                subs.append(sub)

    bokehSec(data=data, subject=subs, fname=fname, ys=ys, xs=xs, zs=zs, variabels=frameVars)
    return


def regulate(lat, lon, depth, data):
    depth = -1* depth 
    deltaZ = np.min( np.abs( depth - np.roll(depth, -1) ) )
    newDepth =  np.arange(np.min(depth), np.max(depth), deltaZ)        

    if len(lon) > len(lat):
        lon1, depth1 = np.meshgrid(lon, depth)
        lon2, depth2 = np.meshgrid(lon, newDepth)
        lon1 = lon1.ravel()
        lon1 = list(lon1[lon1 != np.isnan])
        depth1 = depth1.ravel()
        depth1 = list(depth1[depth1 != np.isnan])
        data = data.ravel()
        data = list(data[data != np.isnan])
        data = griddata((lon1, depth1), data, (lon2, depth2), method='linear')
    else:   
        lat1, depth1 = np.meshgrid(lat, depth)
        lat2, depth2 = np.meshgrid(lat, newDepth)
        lat1 = lat1.ravel()
        lat1 = list(lat1[lat1 != np.isnan])
        depth1 = depth1.ravel()
        depth1 = list(depth1[depth1 != np.isnan])
        data = data.ravel()
        data = list(data[data != np.isnan])
        data = griddata((lat1, depth1), data, (lat2, depth2), method='linear')

    depth = -1* depth 
    return data




def bokehSec(data, subject, fname, ys, xs, zs, variabels):
    TOOLS="hover,crosshair,pan,wheel_zoom,zoom_in,zoom_out,box_zoom,undo,redo,reset,tap,save,box_select,poly_select,lasso_select,"
    w = 1000
    h = 500
    p = []
    data_org = list(data)
    for ind in range(len(data_org)):
        data = data_org[ind]
        lon = xs[ind]
        lat = ys[ind]
        depth = zs[ind]      
        
        bounds = getBounds(variabels[ind])
        bounds = (None, None)
        paletteName = getPalette(variabels[ind], 10)

        if bounds[0]==None:
            color_mapper = LinearColorMapper(palette=paletteName)
        else:   
            color_mapper = LinearColorMapper(palette=paletteName, low=bounds[0], high=bounds[1])     

        if len(lon) > len(lat):
            p1 = figure(tools=TOOLS, toolbar_location="above", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lon), np.max(lon)), y_range=(-np.max(depth), -np.min(depth)))
            data = np.nanmean(data, axis=0)
            data = np.transpose(data)
            data = np.squeeze(data)
            p1.xaxis.axis_label = 'Longitude'
            data = regulate(lat, lon, depth, data)
            p1.image(image=[data], color_mapper=color_mapper, x=np.min(lon), y=-np.max(depth), dw=np.max(lon)-np.min(lon), dh=np.max(depth)-np.min(depth))
        else:
            p1 = figure(tools=TOOLS, toolbar_location="above", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lat), np.max(lat)), y_range=(-np.max(depth), -np.min(depth)))
            data = np.nanmean(data, axis=1)
            data = np.transpose(data)
            data = np.squeeze(data)
            p1.xaxis.axis_label = 'Latitude'      
            data = regulate(lat, lon, depth, data)
            p1.image(image=[data], color_mapper=color_mapper, x=np.min(lat), y=-np.max(depth), dw=np.max(lat)-np.min(lat), dh=np.max(depth)-np.min(depth))


        p1.yaxis.axis_label = 'depth [m]'
        color_bar = ColorBar(color_mapper=color_mapper, ticker=BasicTicker(),
                        label_standoff=12, border_line_color=None, location=(0,0))
        p1.add_layout(color_bar, 'right')
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    if not inline:      ## if jupyter is not the caller
        output_file(dirPath + fname + ".html", title="Section Map")
    show(column(p))
    return


def main():
    tables = sys.argv[1]      
    variables = sys.argv[2]      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]      
    lat1 = sys.argv[5]      
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]      
    lon2 = sys.argv[8]      
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]      
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    sectionMap(tables.split(','), variables.split(','), dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)



inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()    